const db = require('../../../util/db')
const _ = require('underscore')
const path = require('path')
const fs = require('fs')
const Boom = require('boom')
const Cliente = require('../../../models/Cliente');
const ClienteAtendimento = require('../../../models/ClienteAtendimento');
const ClienteNumerosAdicionais = require('../../../models/ClienteNumerosAdicionais');
const ClienteEmailsAdicionais = require('../../../models/ClienteEmailsAdicionais');
const ClienteCamposAdicionais = require('../../../models/ClienteCamposAdicionais')

exports.getContato = async (request, reply) => {
    try {
        const sqlChats = `select json_agg(chats) as chats FROM
        (SELECT 
        a.*,
        ( 
            SELECT json_agg(item)
            FROM (
            SELECT b.*
            FROM cliente b where id = a."idCliente"
            ) item
        ) AS cliente,
        ( 
            SELECT json_agg(item)
            FROM (
            SELECT b.agente,b.obs,c.*
            FROM "queueChatClassificacao" b 
            INNER JOIN "classificacoes" c on b."classificacaoID" = c.id
            where b."queueChatID" = a.id	  
            ) item
        ) AS classificacao,
        ( 
            SELECT json_agg(item)
            FROM (
            SELECT m.*,
            ( 
                SELECT json_agg(item)
                FROM (
                SELECT an.*
                FROM "queueAnexos" an
                where an."id_message" = m.id	  
                ) item
            ) AS anexos
            FROM "queueChatMessages" m
            where m."idQueueChat" = a.id
            ORDER BY m."dataHora" ASC
            ) item
        ) AS mensagens
        FROM "queueChat" a 
        WHERE a."queueTypeID" != 1 AND a."ultimoAgente" = $1 AND a."dataIni" >= $2 AND a."idCliente" = $3
        ORDER BY id ASC
        ) as chats`

        const sqlCalls = `select json_agg(calls) as calls FROM
        (SELECT 
        a.*,a."calldate" as "data_hora_last_msg", 1 as "queueTypeID",
        ( 
            SELECT json_agg(item)
            FROM (
            SELECT b.*
            FROM cliente b where b.id = a."idCliente"
            ) item
        ) AS cliente,
        ( 
            SELECT json_agg(item)
            FROM (
            SELECT b."data",b.obs,b.protocolo,c.*
            FROM "cdr_classificacoes" b 
            INNER JOIN "classificacoes" c on b."id_classificacao" = c.id
            where b."uniqueid" = a."callid"	  
            ) item
        ) AS classificacao
        FROM "queue_calls" a 
        WHERE a."agent" = $1 AND a."calldate" >= $2 AND a."idCliente" = $3
        ORDER BY calldate ASC
        ) as calls`

        let d = new Date()
        if (request.query.dataStart) {
            d = new Date(request.query.dataStart)
        }
        else {
            d.setMonth(d.getMonth() - 3);
            d.setHours(0, 0, 0, 0);
        }

        const retChats = await db.query(sqlChats, [request.params.AgenteFullName, d, request.params.ContatoID])
        const retCall = await db.query(sqlCalls, [request.params.AgenteFullName, d, request.params.ContatoID])

        const ret = _.union(retChats.rows[0].chats, retCall.rows[0].calls)

        const ret2 = _.chain(ret)
            .groupBy('idCliente')
            .sortBy('data_hora_last_msg')
            .map((clientes) => {
                let ob = clientes[0].cliente ? clientes[0].cliente[0] : { naocliente: true }
                let atendimentos = clientes.map(at => {
                    const { cliente, ...dados } = at
                    return dados
                })
                ob.data_hora_last_msg = atendimentos.reduce((a, b) => (new Date(a.data_hora_last_msg).getTime() > new Date(b.data_hora_last_msg).getTime() ? a : b)).data_hora_last_msg;

                const allAtendimentos = _.groupBy(atendimentos, 'queueTypeID')
                ob.atendimentos = Object.entries(allAtendimentos).map(tipo => {
                    const at = {}
                    at.tipo = tipo[0]
                    at.atendimentos = tipo[1]
                    return at
                })
                return ob
            })
            .reverse()
            .value()

        reply(ret2);
    } catch (error) {
        reply(Boom.badData(error));
    }
}

exports.getFotoContato = async (request, reply) => {
    try {
        const contato = await Cliente.fetchById(request.params.ContatoID)
        if (!contato || !contato.profileImage || !fs.existsSync(contato.profileImage)) {
            return reply(Boom.badData(`Arquivo não encontrado`));
        }
        return reply.file(contato.profileImage, {
            name: path.basename(contato.profileImage),
            confine: false,
        });
    } catch (error) {
        console.log(error);
        reply(Boom.badData(error));
    }
};
exports.findCliente = async (request, reply) => {
    try {
        const cliente = await Cliente.BuscaClientePorNumero(request.params.Numero)
        if (cliente[0]) {
            cliente[0].camposAdicionais = await ClienteCamposAdicionais.query.where({ clienteID: cliente[0].id }).fetch();
        }
        console.log('CLIENTE', cliente)
        reply(cliente[0])
    } catch (error) {
        console.log(error);
        reply(Boom.badData(error));
    }
}


exports.uploadFotoContato = async (request, reply) => {
    try {
        const dir = '/gravacoes/clientes/'
        const file_path = path.resolve(dir, `${request.params.ContatoID.toString()}.jpg`)
        if (!fs.existsSync(path.resolve(dir))) {
            fs.mkdirSync(path.resolve(dir), { recursive: true })
        }
        const file = fs.createWriteStream(file_path)
        file.on('error', function (err) {
            console.error(err);
            fs.unlinkSync(file_path);
            reply(Boom.badData(err));
        });
        request.payload.file.pipe(file);
        request.payload.file.on('end', async function (err) {
            if (err) {
                fs.unlinkSync(file_path);
                reply(Boom.badData(err));
            }
            else {
                let dados = {
                    id: request.params.ContatoID,
                    profileImage: file_path
                }
                let ret = await Cliente.update(dados);
                reply(ret);
            }
        })
    } catch (error) {
        console.error(error);
        reply(Boom.badData(error));
    }
}
exports.trocarContatoAtendimento = async (request, reply) => {
    try {
        const atendimento = await ClienteAtendimento.query.where('id', request.params.idAtendimento).update({ idCliente: request.payload.idClienteNovo });
        reply(atendimento);
    } catch (error) {
        console.error(error);
        reply(Boom.badData(error));
    }
}
exports.vincularContato = async (request, reply) => {
    const clienteAntigo = await Cliente.query.require(false).where('id', request.payload.idClienteAntigo).first().fetch();
    const clienteAntigoEmailsAdicionais = await ClienteEmailsAdicionais.query.require(false).where('clienteID', request.payload.idClienteAntigo).fetch();
    const clienteAntigoCamposAdicionais = await ClienteCamposAdicionais.query.require(false).where('clienteID', request.payload.idClienteAntigo).fetch();
    const clienteAntigoNumeroAdicionais = await ClienteNumerosAdicionais.query.require(false).where('clienteID', request.payload.idClienteAntigo).fetch();

    const transaction = new db.orm.Transaction();
    const ClienteT = transaction.models.Cliente;
    const QueueChat = transaction.models.QueueChat;
    let transactionInsert;
    try {
        
        const count = await QueueChat.query.require(false).where('idCliente', request.payload.idClienteNovo).count();
        if (count > 0) {
            //Atualizando o id do cliente escolhido para o cliente vinculado
            await QueueChat.query.where('idCliente', request.payload.idClienteAntigo).update({ idCliente: request.payload.idClienteNovo });
        }
        await ClienteT.query
        .require(false)
        .where({ id: clienteAntigo.id })
        .delete()

        await transaction.commit();

        transactionInsert = new db.orm.Transaction();
        const ClienteNumerosAdicionaisT = transactionInsert.models.ClienteNumerosAdicionais;
        const ClienteEmailsAdicionaisT = transactionInsert.models.ClienteEmailsAdicionais;
        const ClienteCamposAdicionaisT = transactionInsert.models.ClienteCamposAdicionais;

        if (clienteAntigo.numero) await ClienteNumerosAdicionaisT.insert([{ nome: clienteAntigo.nome, numero: clienteAntigo.numero, clienteID: request.payload.idClienteNovo }], { require: false });
        if (clienteAntigo.email) {
            //Criando um email adicional com o email do cliente antigo
            await ClienteEmailsAdicionaisT.insert([{ nome: clienteAntigo.nome, email: clienteAntigo.email, clienteID: request.payload.idClienteNovo }], { require: false });
        }
        if (clienteAntigoEmailsAdicionais) {
            for (let emailAntigo of clienteAntigoEmailsAdicionais) {
                await ClienteEmailsAdicionaisT.insert([{ nome: `${clienteAntigo.nome} ${emailAntigo.nome}` , email: emailAntigo.email, clienteID: request.payload.idClienteNovo }], { require: false });
            }
        }
        if (clienteAntigoNumeroAdicionais) {
            for (let numeroAntigo of clienteAntigoNumeroAdicionais) {
                await ClienteNumerosAdicionaisT.insert([{ nome: `${clienteAntigo.nome} ${numeroAntigo.nome}` , numero: numeroAntigo.numero, clienteID: request.payload.idClienteNovo }], { require: false });
            }
        }
        if (clienteAntigoCamposAdicionais) {
            for (let campoAntigo of clienteAntigoCamposAdicionais) {
                await ClienteCamposAdicionaisT.insert([{ nome: `${clienteAntigo.nome} ${campoAntigo.nome}` , valor: campoAntigo.valor, clienteID: request.payload.idClienteNovo }], { require: false });
            }
        }
        await transactionInsert.commit()

        reply({ error: false, msg: "Cliente vinculado" });

    } catch (error) {
        transaction.rollback();
        transactionInsert.rollback();
        console.error(error);
        reply(Boom.badData(error));
    }
}

exports.vincularContatoNovo = async (request, reply) => {
    const clienteAntigo = await Cliente.query
    .require(false)
    .where('id', request.payload.idClienteAntigo)
    .first()
    .fetch();

    const clienteAntigoEmailsAdicionais = await ClienteEmailsAdicionais.query
    .require(false)
    .where('clienteID', request.payload.idClienteAntigo)
    .fetch();

    const clienteAntigoNumeroAdicionais = await ClienteNumerosAdicionais.query
    .require(false)
    .where('clienteID', request.payload.idClienteAntigo)
    .fetch();

    const clienteAntigoCamposAdicionais = await ClienteCamposAdicionais.query
    .require(false)
    .where('clienteID', request.payload.idClienteAntigo)
    .fetch();

    const transaction = new db.orm.Transaction();
    const ClienteT = transaction.models.Cliente;
    const QueueChat = transaction.models.QueueChat;
    
    try {
        // Primeira transação para deletar cliente e dados adicionais
        const transaction = new db.orm.Transaction();
       

        await QueueChat.query.require(false).where('idCliente', request.payload.idClienteAntigo).update({ idCliente: request.payload.idClienteNovo });

        await ClienteT.query
            .require(false)
            .where({ id: clienteAntigo.id })
            .delete();
        
        await transaction.commit();
        const transactionInsert = new db.orm.Transaction();
        // console.log('SEGUNDA===========')

        // // Segunda transação para inserir dados adicionais
        // const transactionInsert = new db.orm.Transaction();

        // const ClienteNumerosAdicionaisT = transactionInsert.models.ClienteNumerosAdicionais;
        // const ClienteEmailsAdicionaisT = transactionInsert.models.ClienteEmailsAdicionais;
        // const ClienteCamposAdicionaisT = transactionInsert.models.ClienteCamposAdicionais;

        // // Criando um novo número adicional para o cliente vinculado
        // if (clienteAntigo.numero) {
        //     await ClienteNumerosAdicionaisT.insert([{ nome: clienteAntigo.nome + ' ' + clienteAntigo.numero, numero: clienteAntigo.numero, clienteID: request.payload.idClienteNovo }], { require: false });
        // }

        // if (clienteAntigo.email) {
        //     // Criando um email adicional com o email do cliente antigo
        //     await ClienteEmailsAdicionaisT.insert([{ nome: clienteAntigo.nome, email: clienteAntigo.email, clienteID: request.payload.idClienteNovo }], { require: false });
        // }

        // for (let emailAntigo of clienteAntigoEmailsAdicionais) {
        //     await ClienteEmailsAdicionaisT.insert([{ nome: `${clienteAntigo.nome} ${emailAntigo.nome}`, email: emailAntigo.email, clienteID: request.payload.idClienteNovo }], { require: false });
        // }

        // for (let numeroAntigo of clienteAntigoNumeroAdicionais) {
        //     await ClienteNumerosAdicionaisT.insert([{ nome: `${clienteAntigo.nome} ${numeroAntigo.nome}`, numero: numeroAntigo.numero, clienteID: request.payload.idClienteNovo }], { require: false });
        // }

        

        // for (let campoAntigo of clienteAntigoCamposAdicionais) {
        //     await ClienteCamposAdicionaisT.insert([{ nome: `${clienteAntigo.nome} ${campoAntigo.nome}`, valor: campoAntigo.valor, clienteID: request.payload.idClienteNovo }], { require: false });
        // }

        // await transactionInsert.commit();

        reply({ error: false, msg: "Cliente vinculado com sucesso" });

    } catch (error) {
        transaction.rollback();
        console.error(error);
        reply(Boom.badData(error));
    }
}


