const axios = require('axios');
const { parse } = require('node-html-parser');

exports.extractMetadata = async (request, reply) => {
  
  const url = request.payload.url.startsWith('http') || request.payload.url.startsWith('https')  ? request.payload.url : `https://${request.payload.url}`;
  if (!url) {
    return reply({ error: 'URL is required' }).code(400);
  }
  try {
    // Faz a requisição HTTP para o site desejado
    const { data } = await axios.get(url);

    // Faz o parsing do HTML
    const root = parse(data);
    // Extrai os metadados relevantes
    if (root) {
      let title = null
      let description = null
      let ogImage = null
      let ogTitle = null
      let ogDescription = null
      // Título da página
      if (root.querySelector('title')) {
      title = root.querySelector('title').innerText;
    }

    // Descrição padrão (caso não tenha og:description ou twitter:description)
    if (root.querySelector('meta[name="description"]')) {
      description = root.querySelector('meta[name="description"]').getAttribute('content');
    }

    // Imagem OG
    if (root.querySelector('meta[property="og:image"]')) {
      ogImage = root.querySelector('meta[property="og:image"]').getAttribute('content');
    } else if (root.querySelector('meta[name="twitter:image"]')) {
      // Caso não tenha og:image, utiliza a imagem do Twitter
      ogImage = root.querySelector('meta[name="twitter:image"]').getAttribute('content');
    }
    // Transformar a imagem em URL absoluta, se for relativa
    if (ogImage && !ogImage.startsWith('http://') && !ogImage.startsWith('https://') && !ogImage.startsWith('www.')) {
      try {
        const urlCompleta = new URL(ogImage, url);
        ogImage = urlCompleta.href;
      } catch (error) {
        console.error('Erro ao construir a URL da imagem:', error);
      }
    }

    // Título OG
    if (root.querySelector('meta[property="og:title"]')) {
      ogTitle = root.querySelector('meta[property="og:title"]').getAttribute('content');
    } else if (root.querySelector('meta[name="twitter:title"]')) {
      // Caso não tenha og:title, utiliza o título do Twitter
      ogTitle = root.querySelector('meta[name="twitter:title"]').getAttribute('content');
    }

    // Descrição OG
    if (root.querySelector('meta[property="og:description"]')) {
      ogDescription = root.querySelector('meta[property="og:description"]').getAttribute('content');
    } else if (root.querySelector('meta[name="twitter:description"]')) {
      // Caso não tenha og:description, utiliza a descrição do Twitter
      ogDescription = root.querySelector('meta[name="twitter:description"]').getAttribute('content');
    }

    if (ogImage) {
        await axios.get(ogImage).then((response) => {
          if (!(response.status === 200 && response.headers['content-type'].startsWith('image/'))) {
          ogImage = null
          }
        }).catch(() => ogImage = null);
    }
    // Responde com os metadados extraídos
    return reply({
      title: ogTitle || title,
      description: ogDescription || description,
      image: ogImage,
    });
    }
  } catch (error) {
    console.error('Error fetching metadata:', error);
    return reply({ error: 'Failed to fetch metadata' }).code(404);
  }
};